==> jollyday-core-0.22.0-sources.jar: de/focus_shift/jollyday/core/HolidayManager.java <==
package de.focus_shift.jollyday.core;

import de.focus_shift.jollyday.core.caching.Cache;
import de.focus_shift.jollyday.core.caching.HolidayManagerValueHandler;
import de.focus_shift.jollyday.core.configuration.ConfigurationProviderManager;
import de.focus_shift.jollyday.core.parser.functions.CalendarToLocalDate;
import de.focus_shift.jollyday.core.spi.ConfigurationService;
import de.focus_shift.jollyday.core.util.CalendarUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.LocalDate;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

/**
 * Abstract base class for all holiday manager implementations. Upon call of
 * getInstance method the implementing class will be read from the
 * jollyday.properties file and instantiated.
 *
 * @author Sven Diedrichsen
 * @version $Id: $
 */
public abstract class HolidayManager {

  private static final Logger LOG = LoggerFactory.getLogger(HolidayManager.class);

  /**
   * Signifies if caching of manager instances is enabled. If not every call
   * to getInstance will return a newly instantiated and initialized manager.
   */
  private static boolean cachingEnabled = true;
  /**
   * Cache for manager instances on a per country basis.
   */
  private static final Cache<HolidayManager> HOLIDAY_MANAGER_CACHE = new Cache<>();
  /**
   * Manager for configuration providers. Delivers the jollyday configuration.
   */
  private static final ConfigurationProviderManager CONFIGURATION_MANAGER_PROVIDER = new ConfigurationProviderManager();
  /**
   * the holiday cache
   */
  private final Cache<Set<Holiday>> holidayCache = new Cache<>();
  /**
   * Utility for calendar operations
   */
  protected final CalendarUtil calendarUtil = new CalendarUtil();
  /**
   * The datasource to get the holiday data from.
   */
  private ConfigurationService configurationService;
  /**
   * the manager parameter
   */
  private ManagerParameter managerParameter;

  /**
   * Creates a HolidayManager instance for the default locale country using
   * the configured properties from the configuration file.
   *
   * @return a eventually cached HolidayManager instance
   */
  public static HolidayManager getInstance() {
    return getInstance(ManagerParameters.create((String) null, null));
  }

  /**
   * Creates a HolidayManager instance for the default locale country using
   * the provided properties.
   *
   * @param properties the overriding configuration properties.
   * @return a eventually cached HolidayManager instance
   */
  public static HolidayManager getInstance(Properties properties) {
    return getInstance(ManagerParameters.create((String) null, properties));
  }

  /**
   * Creates and returns a {@link HolidayManager} for the provided
   * {@link ManagerParameters}
   *
   * @param parameter the {@link ManagerParameters} to create the manager with
   * @return the {@link HolidayManager} instance
   */
  public static HolidayManager getInstance(ManagerParameter parameter) {
    return createManager(parameter);
  }

  /**
   * Creates a new <code>HolidayManager</code> instance for the country and
   * puts it to the manager cache.
   *
   * @param parameter the parameter will be merged into the current configuration
   * @return created or cached holiday manager
   */
  private static HolidayManager createManager(final ManagerParameter parameter) {
    LOG.debug("Creating HolidayManager for calendar '{}'. Caching enabled: {}", parameter, isManagerCachingEnabled());
    CONFIGURATION_MANAGER_PROVIDER.mergeConfigurationProperties(parameter);
    final String managerImplClassName = readManagerImplClassName(parameter);
    final HolidayManagerValueHandler holidayManagerValueHandler = new HolidayManagerValueHandler(parameter, managerImplClassName);
    if (isManagerCachingEnabled()) {
      return HOLIDAY_MANAGER_CACHE.get(holidayManagerValueHandler);
    } else {
      return holidayManagerValueHandler.createValue();
    }
  }

  /**
   * Reads the managers implementation class from the properties config file.
   *
   * @param parameter the parameter to read the manager implementation class from
   * @return the manager implementation class name
   */
  private static String readManagerImplClassName(ManagerParameter parameter) {
    final String className = parameter.getManagerImplClassName();
    if (className == null) {
      throw new IllegalStateException("Missing configuration '" + ManagerParameter.MANAGER_IMPL_CLASS_PREFIX + "'. Cannot create manager.");
    }
    return className;
  }

  /**
   * If true, instantiated managers will be cached. If false every call to
   * getInstance will create new manager. True by default.
   *
   * @param managerCachingEnabled the managerCachingEnabled to set
   */
  public static void setManagerCachingEnabled(boolean managerCachingEnabled) {
    cachingEnabled = managerCachingEnabled;
  }

  /**
   * <p>
   * isManagerCachingEnabled.
   * </p>
   *
   * @return the CACHING_ENABLED
   */
  public static boolean isManagerCachingEnabled() {
    return cachingEnabled;
  }

  /**
   * Clears the manager cache from all cached manager instances.
   */
  public static void clearManagerCache() {
    HOLIDAY_MANAGER_CACHE.clear();
  }

  /**
   * Calls isHoliday with {@link LocalDate} object.
   *
   * @param c           {@link java.util.Calendar} to check.
   * @param holidayType type holidays to be considered. NULL checks any.
   * @param args        a {@link java.lang.String} object.
   * @return if the date is a holiday
   */
  public boolean isHoliday(final Calendar c, HolidayType holidayType, final String... args) {
    return isHoliday(new CalendarToLocalDate().apply(c), holidayType, args);
  }

  public boolean isHoliday(final Calendar c, final String... args) {
    return isHoliday(c, null, args);
  }

  /**
   * Show if the requested date is a holiday.
   *
   * @param c           The potential holiday.
   * @param holidayType a {@link HolidayType} object
   * @param args        Hierarchy to request the holidays for. i.e. args = {'ny'} -&gt;
   *                    New York holidays
   * @return is a holiday in the state/region
   */
  public boolean isHoliday(final LocalDate c, HolidayType holidayType, final String... args) {
    final StringBuilder keyBuilder = new StringBuilder();
    keyBuilder.append(c.getYear());
    for (String arg : args) {
      keyBuilder.append("_");
      keyBuilder.append(arg);
    }
    Set<Holiday> holidays = holidayCache.get(new Cache.ValueHandler<Set<Holiday>>() {
      @Override
      public String getKey() {
        return keyBuilder.toString();
      }

      @Override
      public Set<Holiday> createValue() {
        return getHolidays(c.getYear(), args);
      }
    });
    return calendarUtil.contains(holidays, c, holidayType);
  }

  /**
   * Calls #isHoliday(c, null, args)
   *
   * @param c    the date to check
   * @param args the arguments to find the calendar
   * @return whether the date is a holiday
   */
  public boolean isHoliday(final LocalDate c, final String... args) {
    return isHoliday(c, null, args);
  }

  /**
   * Returns a set of all currently supported calendar codes.
   *
   * @return Set of supported calendar codes.
   */
  public static Set<String> getSupportedCalendarCodes() {
    Set<String> supportedCalendars = new HashSet<>();
    for (HolidayCalendar c : HolidayCalendar.values()) {
      supportedCalendars.add(c.getId());
    }
    return supportedCalendars;
  }

  /**
   * Sets the configuration datasource with this holiday manager.
   *
   * @param configurationService the {@link ConfigurationService} to use.
   */
  public void setConfigurationService(ConfigurationService configurationService) {
    this.configurationService = configurationService;
  }

  /**
   * Returns the {@link ConfigurationService} to be used to retrieve
   * holiday data.
   *
   * @return the {@link ConfigurationService} to use.
   */
  public ConfigurationService getConfigurationService() {
    return configurationService;
  }

  public ManagerParameter getManagerParameter() {
    return managerParameter;
  }

  /**
   * Initializes the implementing manager for the provided calendar.
   *
   * @param parameters i.e. us, uk, de
   */
  public void init(ManagerParameter parameters) {
    this.managerParameter = parameters;
    this.doInit();
  }

  public abstract void doInit();

  /**
   * Returns the holidays for the requested year and hierarchy structure.
   *
   * @param year i.e. 2010
   * @param args i.e. args = {'ny'}. returns US/New York holidays. No args means holidays common to whole country
   * @return a set of holidays for the requested year
   */
  public abstract Set<Holiday> getHolidays(int year, String... args);

  /**
   * Returns the holidays for the requested interval and hierarchy structure.
   *
   * @param startDateInclusive the start date of the interval in which holidays lie, inclusive
   * @param endDateInclusive   the end date of the interval in which holidays lie, inclusive
   * @param args               a {@link java.lang.String} object.
   * @return list of holidays within the interval
   */
  public abstract Set<Holiday> getHolidays(LocalDate startDateInclusive,
                                           LocalDate endDateInclusive, String... args);

  /**
   * Returns the configured hierarchy structure for the specific manager. This
   * hierarchy shows how the configured holidays are structured and can be
   * retrieved.
   *
   * @return Current calendars hierarchy
   */
  public abstract CalendarHierarchy getCalendarHierarchy();

}

==> jollyday-core-0.25.0-sources.jar: de/focus_shift/jollyday/core/HolidayManager.java <==
package de.focus_shift.jollyday.core;

import de.focus_shift.jollyday.core.caching.Cache;
import de.focus_shift.jollyday.core.caching.HolidayManagerValueHandler;
import de.focus_shift.jollyday.core.configuration.ConfigurationProviderManager;
import de.focus_shift.jollyday.core.datasource.ConfigurationServiceManager;
import de.focus_shift.jollyday.core.parser.functions.CalendarToLocalDate;
import de.focus_shift.jollyday.core.spi.ConfigurationService;
import de.focus_shift.jollyday.core.support.LazyServiceLoaderCache;
import de.focus_shift.jollyday.core.util.CalendarUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.LocalDate;
import java.util.Calendar;
import java.util.Properties;
import java.util.Set;

import static java.util.Arrays.stream;
import static java.util.stream.Collectors.toUnmodifiableSet;

/**
 * Abstract base class for all holiday manager implementations. Upon call of
 * getInstance method the implementing class will be read from the
 * jollyday.properties file and instantiated.
 */
public abstract class HolidayManager {

  private static final Logger LOG = LoggerFactory.getLogger(HolidayManager.class);

  /**
   * Signifies if caching of manager instances is enabled. If not every call
   * to getInstance will return a newly instantiated and initialized manager.
   */
  private static boolean cachingEnabled = true;

  /**
   * Cache for manager instances on a per country basis.
   */
  private static final Cache<HolidayManager> HOLIDAY_MANAGER_CACHE = new Cache<>();

  /**
   * Manager for configuration providers. Delivers the jollyday configuration.
   */
  private static final ConfigurationProviderManager CONFIGURATION_MANAGER_PROVIDER = new ConfigurationProviderManager();

  /**
   * Manager for providing configuration data sources which return the holiday
   * data.
   */
  private static final ConfigurationServiceManager configurationServiceManager =
    new ConfigurationServiceManager(new LazyServiceLoaderCache<>(ConfigurationService.class));

  /**
   * Caches the instance based country specific holidays so that e.g.
   * the created HolidayManager via
   * {@code HolidayManager.getInstance(create("de"))} only contains the german holidays.
   * <p>
   * This is also the reason this cache is not static.
   * If it was static all holidays over all holiday manager instances would be cached,
   * but only the german holidays are important for the german holiday manager, so only save them.
   */
  private final Cache<Set<Holiday>> holidayCache = new Cache<>();

  /**
   * The datasource to get the holiday data from.
   */
  private ConfigurationService configurationService;

  /**
   * the manager parameter
   */
  private ManagerParameter managerParameter;

  /**
   * Initializes the implementing manager for the provided calendar.
   *
   * @param parameters i.e. us, uk, de
   */
  public void init(ManagerParameter parameters) {
    this.managerParameter = parameters;
    this.doInit();
  }

  public abstract void doInit();

  /**
   * Creates a HolidayManager instance for the default locale country using
   * the configured properties from the configuration file.
   *
   * @return an eventually cached HolidayManager instance
   */
  public static HolidayManager getInstance() {
    return getInstance(ManagerParameters.create((String) null, null));
  }

  /**
   * Creates a HolidayManager instance for the default locale country using
   * the provided properties.
   *
   * @param properties the overriding configuration properties.
   * @return an eventually cached HolidayManager instance
   */
  public static HolidayManager getInstance(Properties properties) {
    return getInstance(ManagerParameters.create((String) null, properties));
  }

  /**
   * Creates and returns a {@link HolidayManager} for the provided
   * {@link ManagerParameters}
   *
   * @param parameter the {@link ManagerParameters} to create the manager with
   * @return the {@link HolidayManager} instance
   */
  public static HolidayManager getInstance(ManagerParameter parameter) {
    return createManager(parameter);
  }

  /**
   * Creates a new {@code HolidayManager} instance for the country and
   * puts it to the manager cache.
   *
   * @param parameter the parameter will be merged into the current configuration
   * @return created or cached holiday manager
   */
  private static HolidayManager createManager(final ManagerParameter parameter) {
    LOG.debug("Creating HolidayManager for calendar '{}'. Caching enabled: {}", parameter, isManagerCachingEnabled());
    CONFIGURATION_MANAGER_PROVIDER.mergeConfigurationProperties(parameter);
    final String managerImplClassName = readManagerImplClassName(parameter);
    final HolidayManagerValueHandler holidayManagerValueHandler = new HolidayManagerValueHandler(parameter, managerImplClassName, configurationServiceManager);
    if (isManagerCachingEnabled()) {
      return HOLIDAY_MANAGER_CACHE.get(holidayManagerValueHandler);
    } else {
      return holidayManagerValueHandler.createValue();
    }
  }

  /**
   * Reads the managers implementation class from the properties config file.
   *
   * @param parameter the parameter to read the manager implementation class from
   * @return the manager implementation class name
   */
  private static String readManagerImplClassName(ManagerParameter parameter) {
    final String className = parameter.getManagerImplClassName();
    if (className == null) {
      throw new IllegalStateException("Missing configuration '" + ManagerParameter.MANAGER_IMPL_CLASS_PREFIX + "'. Cannot create manager.");
    }
    return className;
  }

  /**
   * If true, instantiated managers will be cached. If false every call to
   * getInstance will create new manager. True by default.
   *
   * @param managerCachingEnabled the managerCachingEnabled to set
   */
  public static void setManagerCachingEnabled(boolean managerCachingEnabled) {
    cachingEnabled = managerCachingEnabled;
  }

  /**
   * <p>
   * isManagerCachingEnabled.
   * </p>
   *
   * @return the CACHING_ENABLED
   */
  public static boolean isManagerCachingEnabled() {
    return cachingEnabled;
  }

  /**
   * Clears the manager cache from all cached manager instances.
   */
  public static void clearManagerCache() {
    HOLIDAY_MANAGER_CACHE.clear();
  }

  /**
   * Returns true or false if the requested calendar date is a holiday in the state or
   * the optional given region and below
   *
   * @param calendar {@link java.util.Calendar} to check.
   * @param args     Hierarchy to request the holidays for. i.e. args = {'ny'} -&gt; New York holidays
   * @return if the date is a holiday
   */
  public boolean isHoliday(final Calendar calendar, final String... args) {
    return isHoliday(calendar, null, args);
  }

  /**
   * Returns true or false if the requested calendar date is a holiday in the state or
   * based on the given {@link HolidayType} and
   * the optional given region and below
   *
   * @param calendar    {@link java.util.Calendar} to check.
   * @param holidayType a {@link HolidayType} to be considered
   * @param args        Hierarchy to request the holidays for. i.e. args = {'ny'} -&gt; New York holidays
   * @return if the date is a holiday
   */
  public boolean isHoliday(final Calendar calendar, HolidayType holidayType, final String... args) {
    return isHoliday(new CalendarToLocalDate().apply(calendar), holidayType, args);
  }

  /**
   * Returns true or false if the requested date is a holiday in the state or
   * the optional given region and below
   *
   * @param localDate The potential holiday.
   * @param args      Hierarchy to request the holidays for. i.e. args = {'ny'} -&gt; New York holidays
   * @return true if the given date is a holiday in the state/region and below, otherwise false
   */
  public boolean isHoliday(final LocalDate localDate, final String... args) {
    return isHoliday(localDate, null, args);
  }

  /**
   * Returns true or false if the requested date is a holiday in the state,
   * based on the given {@link HolidayType} and
   * the optional given region and below
   *
   * @param localDate   The potential holiday.
   * @param holidayType a {@link HolidayType} to be considered
   * @param args        Hierarchy to request the holidays for. i.e. args = {'ny'} -&gt; New York holidays
   * @return true if the given date is a holiday in the state/region and below, otherwise false
   */
  public boolean isHoliday(final LocalDate localDate, HolidayType holidayType, final String... args) {

    final StringBuilder keyBuilder = new StringBuilder();
    keyBuilder.append(localDate.getYear());
    for (String arg : args) {
      keyBuilder.append("_");
      keyBuilder.append(arg);
    }

    final Cache.ValueHandler<Set<Holiday>> valueHandler = new Cache.ValueHandler<>() {
      @Override
      public String getKey() {
        return keyBuilder.toString();
      }

      @Override
      public Set<Holiday> createValue() {
        return getHolidays(localDate.getYear(), args);
      }
    };

    final Set<Holiday> holidays = holidayCache.get(valueHandler);
    return CalendarUtil.contains(holidays, localDate, holidayType);
  }

  /**
   * Returns a set of all currently supported ISO 3166-1 alpha-2 codes.
   *
   * @return Set of supported calendar codes.
   */
  public static Set<String> getSupportedCalendarCodes() {
    // TODO move this into HolidayCalendar class
    return stream(HolidayCalendar.values())
      .map(HolidayCalendar::getId)
      .collect(toUnmodifiableSet());
  }

  /**
   * Sets the configuration datasource with this holiday manager.
   *
   * @param configurationService the {@link ConfigurationService} to use.
   */
  public void setConfigurationService(ConfigurationService configurationService) {
    this.configurationService = configurationService;
  }

  /**
   * Returns the {@link ConfigurationService} to be used to retrieve
   * holiday data.
   *
   * @return the {@link ConfigurationService} to use.
   */
  public ConfigurationService getConfigurationService() {
    return configurationService;
  }

  /**
   * Returns the {@link ManagerParameter} to be used
   *
   * @return the {@link ManagerParameter} to use.
   */
  public ManagerParameter getManagerParameter() {
    return managerParameter;
  }

  /**
   * Returns the holidays for the requested year and hierarchy structure.
   *
   * @param year i.e. 2010
   * @param args i.e. args = {'ny'}. returns US/New York holidays. No args means holidays common to whole country
   * @return a set of holidays for the requested year
   */
  public abstract Set<Holiday> getHolidays(int year, String... args);

  /**
   * Returns the holidays for the requested interval and hierarchy structure.
   *
   * @param startDateInclusive the start date of the interval in which holidays lie, inclusive
   * @param endDateInclusive   the end date of the interval in which holidays lie, inclusive
   * @param args               i.e. args = {'ny'}. returns US/New York holidays. No args means holidays common to whole country
   * @return set of holidays within the interval
   */
  public abstract Set<Holiday> getHolidays(LocalDate startDateInclusive, LocalDate endDateInclusive, String... args);

  /**
   * Returns the configured hierarchy structure for the specific manager. This
   * hierarchy shows how the configured holidays are structured and can be
   * retrieved.
   *
   * @return Current calendars hierarchy
   */
  public abstract CalendarHierarchy getCalendarHierarchy();

}
